@@ -1,29 +1,43 @@
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
-import collections
 import contextlib
+import enum
 import os
+from typing import Generator
+from typing import MutableMapping
+from typing import NamedTuple
+from typing import Optional
+from typing import Tuple
+from typing import Union
 
 
-UNSET = collections.namedtuple('UNSET', ())()
+class _Unset(enum.Enum):
+    UNSET = 1
 
 
-Var = collections.namedtuple('Var', ('name', 'default'))
-setattr(Var.__new__, '__defaults__', ('',))
+UNSET = _Unset.UNSET
 
 
-def format_env(parts, env):
+class Var(NamedTuple):
+    name: str
+    default: str = ''
+
+
+SubstitutionT = Tuple[Union[str, Var], ...]
+ValueT = Union[str, _Unset, SubstitutionT]
+PatchesT = Tuple[Tuple[str, ValueT], ...]
+
+
+def format_env(parts: SubstitutionT, env: MutableMapping[str, str]) -> str:
     return ''.join(
-        env.get(part.name, part.default)
-        if isinstance(part, Var)
-        else part
+        env.get(part.name, part.default) if isinstance(part, Var) else part
         for part in parts
     )
 
 
 @contextlib.contextmanager
-def envcontext(patch, _env=None):
+def envcontext(
+        patch: PatchesT,
+        _env: Optional[MutableMapping[str, str]] = None,
+) -> Generator[None, None, None]:
     """In this context, `os.environ` is modified according to `patch`.
 
     `patch` is an iterable of 2-tuples (key, value):
@@ -35,7 +49,7 @@ def envcontext(patch, _env=None):
               replaced with the previous environment
     """
     env = os.environ if _env is None else _env
-    before = env.copy()
+    before = dict(env)
 
     for k, v in patch:
         if v is UNSET:
